@@ -21,23 +21,20 @@
 #  include <config.h>
 #endif
 
+#include <algorithm>
 #include <string>
 #include <vector>
 
+#include <simgear/sg_inlines.h>
 #include <simgear/math/SGMath.hxx>
-#include <simgear/math/point3d.hxx>
 #include <simgear/math/sg_geodesy.hxx>
+
 #include <math.h>
 #include <Main/util.hxx>
 #include <Main/viewer.hxx>
 
 #include "AICarrier.hxx"
 
-/** Value of earth radius (meters) */
-#define RADIUS_M   SG_EQUATORIAL_RADIUS_M
-
-
-
 FGAICarrier::FGAICarrier() : FGAIShip(otCarrier) {
 }
 
@@ -59,6 +56,7 @@ void FGAICarrier::readFromScenario(SGPropertyNode* scFileNode) {
   setMinLat(scFileNode->getDoubleValue("min-lat", 0));
   setMaxLong(scFileNode->getDoubleValue("max-long", 0));
   setMinLong(scFileNode->getDoubleValue("min-long", 0));
+  setMPControl(scFileNode->getBoolValue("mp-control", false));
 
   SGPropertyNode* flols = scFileNode->getChild("flols-pos");
   if (flols) {
@@ -71,29 +69,8 @@ void FGAICarrier::readFromScenario(SGPropertyNode* scFileNode) {
   } else
     flols_off = SGVec3d::zeros();
 
-  std::vector<SGPropertyNode_ptr> props = scFileNode->getChildren("wire");
+  std::vector<SGPropertyNode_ptr> props = scFileNode->getChildren("parking-pos");
   std::vector<SGPropertyNode_ptr>::const_iterator it;
-  for (it = props.begin(); it != props.end(); ++it) {
-    std::string s = (*it)->getStringValue();
-    if (!s.empty())
-      wire_objects.push_back(s);
-  }
-
-  props = scFileNode->getChildren("catapult");
-  for (it = props.begin(); it != props.end(); ++it) {
-    std::string s = (*it)->getStringValue();
-    if (!s.empty())
-      catapult_objects.push_back(s);
-  }
-
-  props = scFileNode->getChildren("solid");
-  for (it = props.begin(); it != props.end(); ++it) {
-    std::string s = (*it)->getStringValue();
-    if (!s.empty())
-      solid_objects.push_back(s);
-  }
-
-  props = scFileNode->getChildren("parking-pos");
   for (it = props.begin(); it != props.end(); ++it) {
     string name = (*it)->getStringValue("name", "unnamed");
     // Transform to the right coordinate frame, configuration is done in
@@ -140,81 +117,50 @@ void FGAICarrier::setTACANChannelID(const string& id) {
     TACAN_channel_id = id;
 }
 
-void FGAICarrier::getVelocityWrtEarth(sgdVec3& v, sgdVec3& omega, sgdVec3& pivot) {
-    sgdCopyVec3(v, vel_wrt_earth.sg() );
-    sgdCopyVec3(omega, rot_wrt_earth.sg() );
-    sgdCopyVec3(pivot, rot_pivot_wrt_earth.sg() );
+void FGAICarrier::setMPControl(bool c) {
+    MPControl = c;
 }
 
 void FGAICarrier::update(double dt) {
-    // For computation of rotation speeds we just use finite differences here.
-    // That is perfectly valid since this thing is not driven by accelerations
-    // but by just apply discrete changes at its velocity variables.
-    // Update the velocity information stored in those nodes.
+    // Now update the position and heading. This will compute new hdg and
+    // roll values required for the rotation speed computation.
+    FGAIShip::update(dt);
+
+    //automatic turn into wind with a target wind of 25 kts otd
+    //SG_LOG(SG_GENERAL, SG_ALERT, "AICarrier: MPControl " << MPControl << " AIControl " << AIControl);
+    if (!MPControl && AIControl){
+
+        if(turn_to_launch_hdg){
+            TurnToLaunch();
+        } else if(turn_to_recovery_hdg ){
+            TurnToRecover();
+        } else if(OutsideBox() || returning ) {// check that the carrier is inside 
+            ReturnToBox();                     // the operating box,  
+        } else {
+            TurnToBase();
+        }
+
+    } else {
+        FGAIShip::TurnTo(tgt_heading);
+        FGAIShip::AccelTo(tgt_speed);
+    }
+
+    UpdateWind(dt);
+    UpdateElevator(dt, transition_time);
+    UpdateJBD(dt, jbd_transition_time);
+
     // Transform that one to the horizontal local coordinate system.
-    SGQuatd ec2hl = SGQuatd::fromLonLatDeg(pos.lon(), pos.lat());
+    SGQuatd ec2hl = SGQuatd::fromLonLat(pos);
     // The orientation of the carrier wrt the horizontal local frame
     SGQuatd hl2body = SGQuatd::fromYawPitchRollDeg(hdg, pitch, roll);
     // and postrotate the orientation of the AIModel wrt the horizontal
     // local frame
     SGQuatd ec2body = ec2hl*hl2body;
     // The cartesian position of the carrier in the wgs84 world
-    SGVec3d cartPos = SGGeod::fromDegFt(pos.lon(), pos.lat(), pos.elev());
-    // Store for later use by the groundcache
-    rot_pivot_wrt_earth = cartPos;
-
-    // Compute the velocity in m/s in the earth centered coordinate system axis
-    double v_north = 0.51444444*speed*cos(hdg * SGD_DEGREES_TO_RADIANS);
-    double v_east  = 0.51444444*speed*sin(hdg * SGD_DEGREES_TO_RADIANS);
-    vel_wrt_earth = ec2hl.backTransform(SGVec3d(v_north, v_east, 0));
-
-    // Now update the position and heading. This will compute new hdg and
-    // roll values required for the rotation speed computation.
-    FGAIShip::update(dt);
-
-
-    //automatic turn into wind with a target wind of 25 kts otd
-    if(turn_to_launch_hdg){
-        TurnToLaunch();
-    } else if(OutsideBox() || returning) {// check that the carrier is inside the operating box
-        ReturnToBox();
-    } else {
-        TurnToBase();
-    }
-
-    // Only change these values if we are able to compute them safely
-    if (dt < DBL_MIN)
-      rot_wrt_earth = SGVec3d::zeros();
-    else {
-      // Now here is the finite difference ...
-
-      // Transform that one to the horizontal local coordinate system.
-      SGQuatd ec2hlNew = SGQuatd::fromLonLatDeg(pos.lon(), pos.lat());
-      // compute the new orientation
-      SGQuatd hl2bodyNew = SGQuatd::fromYawPitchRollDeg(hdg, pitch, roll);
-      // The rotation difference
-      SGQuatd dOr = inverse(ec2body)*ec2hlNew*hl2bodyNew;
-      SGVec3d dOrAngleAxis;
-      dOr.getAngleAxis(dOrAngleAxis);
-      // divided by the time difference provides a rotation speed vector
-      dOrAngleAxis /= dt;
-
-      // now rotate the rotation speed vector back into the
-      // earth centered frames coordinates
-      dOrAngleAxis = ec2body.backTransform(dOrAngleAxis);
-//       dOrAngleAxis = hl2body.backTransform(dOrAngleAxis);
-//       dOrAngleAxis(1) = 0;
-//       dOrAngleAxis = ec2hl.backTransform(dOrAngleAxis);
-      rot_wrt_earth = dOrAngleAxis;
-    }
-
-    UpdateWind(dt);
-    UpdateElevator(dt, transition_time);
-    UpdateJBD(dt, jbd_transition_time);
-    // For the flols reuse some computations done above ...
+    SGVec3d cartPos = SGVec3d::fromGeod(pos);
 
     // The position of the eyepoint - at least near that ...
-    SGVec3d eyePos(globals->get_current_view()->get_absolute_view_pos());
+    SGVec3d eyePos(globals->get_current_view()->get_view_pos());
     // Add the position offset of the AIModel to gain the earth
     // centered position
     SGVec3d eyeWrtCarrier = eyePos - cartPos;
@@ -252,26 +198,10 @@ void FGAICarrier::update(double dt) {
       source = 0;
 } //end update
 
-bool FGAICarrier::init() {
-    if (!FGAIShip::init())
+bool FGAICarrier::init(bool search_in_AI_path) {
+    if (!FGAIShip::init(search_in_AI_path))
         return false;
 
-    // process the 3d model here
-    // mark some objects solid, mark the wires ...
-
-    // The model should be used for altitude computations.
-    // To avoid that every detail in a carrier 3D model will end into
-    // the aircraft local cache, only set the HOT traversal bit on
-    // selected objects.
-    ssgEntity *sel = aip.getSceneGraph();
-    // Clear the HOT traversal flag
-    mark_nohot(sel);
-    // Selectively set that flag again for wires/cats/solid objects.
-    // Attach a pointer to this carrier class to those objects.
-    mark_wires(sel, wire_objects);
-    mark_cat(sel, catapult_objects);
-    mark_solid(sel, solid_objects);
-
     _longitude_node = fgGetNode("/position/longitude-deg", true);
     _latitude_node = fgGetNode("/position/latitude-deg", true);
     _altitude_node = fgGetNode("/position/altitude-ft", true);
@@ -285,13 +215,16 @@ bool FGAICarrier::init() {
 
 
     turn_to_launch_hdg = false;
+    turn_to_recovery_hdg = false;
+    turn_to_base_course = true;
     returning = false;
+    in_to_wind = false;
 
-    initialpos = pos;
+    mOpBoxPos = pos;
     base_course = hdg;
     base_speed = speed;
 
-    pos_norm = 0;
+    pos_norm = raw_pos_norm = 0;
     elevators = false;
     transition_time = 150;
     time_constant = 0.005;
@@ -322,11 +255,13 @@ void FGAICarrier::bind() {
     props->tie("controls/base-speed-kts",
                 SGRawValuePointer<double>(&base_speed));
     props->tie("controls/start-pos-lat-deg",
-                SGRawValuePointer<double>(&initialpos[1]));
+               SGRawValueMethods<SGGeod,double>(pos, &SGGeod::getLatitudeDeg));
     props->tie("controls/start-pos-long-deg",
-                SGRawValuePointer<double>(&initialpos[0]));
-    props->tie("velocities/speed-kts",
-                SGRawValuePointer<double>(&speed));
+               SGRawValueMethods<SGGeod,double>(pos, &SGGeod::getLongitudeDeg));
+    props->tie("controls/mp-control",
+                SGRawValuePointer<bool>(&MPControl));
+    props->tie("controls/ai-control",
+                SGRawValuePointer<bool>(&AIControl));
     props->tie("environment/surface-wind-speed-true-kts",
                 SGRawValuePointer<double>(&wind_speed_kts));
     props->tie("environment/surface-wind-from-true-degs",
@@ -337,24 +272,31 @@ void FGAICarrier::bind() {
                 SGRawValuePointer<double>(&rel_wind));
     props->tie("environment/rel-wind-speed-kts",
                 SGRawValuePointer<double>(&rel_wind_speed_kts));
-    props->tie("controls/flols/wave-off-lights",
-                SGRawValuePointer<bool>(&wave_off_lights));
+    props->tie("environment/in-to-wind",
+        SGRawValuePointer<bool>(&in_to_wind));
+    //props->tie("controls/flols/wave-off-lights",
+    //            SGRawValuePointer<bool>(&wave_off_lights));
     props->tie("controls/elevators",
                 SGRawValuePointer<bool>(&elevators));
     props->tie("surface-positions/elevators-pos-norm",
                 SGRawValuePointer<double>(&pos_norm));
-    props->tie("controls/elevators-trans-time-s",
+    props->tie("controls/constants/elevators/trans-time-s",
                 SGRawValuePointer<double>(&transition_time));
-    props->tie("controls/elevators-time-constant",
+    props->tie("controls/constants/elevators/time-constant",
                 SGRawValuePointer<double>(&time_constant));
     props->tie("controls/jbd",
         SGRawValuePointer<bool>(&jbd));
     props->tie("surface-positions/jbd-pos-norm",
         SGRawValuePointer<double>(&jbd_pos_norm));
-    props->tie("controls/jbd-trans-time-s",
+    props->tie("controls/constants/jbd/trans-time-s",
         SGRawValuePointer<double>(&jbd_transition_time));
-    props->tie("controls/jbd-time-constant",
+    props->tie("controls/constants/jbd/time-constant",
         SGRawValuePointer<double>(&jbd_time_constant));
+    props->tie("controls/turn-to-recovery-hdg",
+        SGRawValuePointer<bool>(&turn_to_recovery_hdg));
+    props->tie("controls/turn-to-base-course",
+        SGRawValuePointer<bool>(&turn_to_base_course));
+
 
     props->setBoolValue("controls/flols/cut-lights", false);
     props->setBoolValue("controls/flols/wave-off-lights", false);
@@ -362,6 +304,8 @@ void FGAICarrier::bind() {
     props->setBoolValue("controls/crew", false);
     props->setStringValue("navaids/tacan/channel-ID", TACAN_channel_id.c_str());
     props->setStringValue("sign", sign.c_str());
+    props->setBoolValue("controls/lighting/deck-lights", false);
+    props->setDoubleValue("controls/lighting/flood-lights-red-norm", 0);
 }
 
 
@@ -373,21 +317,24 @@ void FGAICarrier::unbind() {
     props->untie("controls/flols/distance-m");
     props->untie("controls/flols/angle-degs");
     props->untie("controls/turn-to-launch-hdg");
-    props->untie("velocities/speed-kts");
     props->untie("environment/wind-speed-true-kts");
     props->untie("environment/wind-from-true-degs");
     props->untie("environment/rel-wind-from-degs");
     props->untie("environment/rel-wind-speed-kts");
-    props->untie("controls/flols/wave-off-lights");
+    props->untie("environment/in-to-wind");
+    //props->untie("controls/flols/wave-off-lights");
     props->untie("controls/elevators");
     props->untie("surface-positions/elevators-pos-norm");
-    props->untie("controls/elevators-trans-time-secs");
-    props->untie("controls/elevators-time-constant");
+    props->untie("controls/constants/elevators/trans-time-secs");
+    props->untie("controls/constants/elevators/time-constant");
     props->untie("controls/jbd");
-    props->untie("surface-positions/jbd-pos-norm");
-    props->untie("controls/jbd-trans-time-s");
+    props->untie("surface-positions/jbd/pos-norm");
+    props->untie("controls/constants/jbd/trans-time-s");
     props->untie("controls/jbd-time-constant");
-
+    props->untie("controls/mp-control");
+    props->untie("controls/ai-control");
+    props->untie("controls/turn-to-recovery-hdg");
+    props->untie("controls/turn-to-base-course");
 }
 
 
@@ -402,7 +349,7 @@ bool FGAICarrier::getParkPosition(const string& id, SGGeod& geodPos,
         if ((*it).name == id || id.empty()) {
             ParkPosition ppos = *it;
             SGVec3d cartPos = getCartPosAt(ppos.offset);
-            geodPos = cartPos;
+            geodPos = SGGeod::fromCart(cartPos);
             hdng = hdg + ppos.heading_deg;
             double shdng = sin(ppos.heading_deg * SGD_DEGREES_TO_RADIANS);
             double chdng = cos(ppos.heading_deg * SGD_DEGREES_TO_RADIANS);
@@ -416,201 +363,9 @@ bool FGAICarrier::getParkPosition(const string& id, SGGeod& geodPos,
     return false;
 }
 
-
-void FGAICarrier::mark_nohot(ssgEntity* e) {
-    if (e->isAKindOf(ssgTypeBranch())) {
-        ssgBranch* br = (ssgBranch*)e;
-        ssgEntity* kid;
-        for ( kid = br->getKid(0); kid != NULL ; kid = br->getNextKid() )
-            mark_nohot(kid);
-
-        br->clrTraversalMaskBits(SSGTRAV_HOT);
-
-    } else if (e->isAKindOf(ssgTypeLeaf())) {
-
-        e->clrTraversalMaskBits(SSGTRAV_HOT);
-    }
-}
-
-
-bool FGAICarrier::mark_wires(ssgEntity* e, const list<string>& wire_objects, bool mark) {
-    bool found = false;
-    if (e->isAKindOf(ssgTypeBranch())) {
-        ssgBranch* br = (ssgBranch*)e;
-        ssgEntity* kid;
-
-        list<string>::const_iterator it;
-        for (it = wire_objects.begin(); it != wire_objects.end(); ++it)
-            mark = mark || (e->getName() && (*it) == e->getName());
-
-        for ( kid = br->getKid(0); kid != NULL ; kid = br->getNextKid() )
-            found = mark_wires(kid, wire_objects, mark) || found;
-
-        if (found)
-            br->setTraversalMaskBits(SSGTRAV_HOT);
-
-    } else if (e->isAKindOf(ssgTypeLeaf())) {
-        list<string>::const_iterator it;
-        for (it = wire_objects.begin(); it != wire_objects.end(); ++it) {
-            if (mark || (e->getName() && (*it) == e->getName())) {
-                e->setTraversalMaskBits(SSGTRAV_HOT);
-                ssgBase* ud = e->getUserData();
-                if (ud) {
-                    FGAICarrierHardware* ch = dynamic_cast<FGAICarrierHardware*>(ud);
-                    if (ch) {
-                        SG_LOG(SG_GENERAL, SG_WARN,
-                                "AICarrier: Carrier hardware gets marked twice!\n"
-                                "           You have probably a whole branch marked as"
-                                " a wire which also includes other carrier hardware.");
-                    } else {
-                        SG_LOG(SG_GENERAL, SG_ALERT,
-                                "AICarrier: Found user data attached to a leaf node which "
-                                "should be marked as a wire!\n    ****Skipping!****");
-                    }
-                } else {
-                    e->setUserData( FGAICarrierHardware::newWire( this ) );
-                    ssgLeaf *l = (ssgLeaf*)e;
-                    if ( l->getNumLines() != 1 ) {
-                        SG_LOG(SG_GENERAL, SG_ALERT,
-                                "AICarrier: Found wires not modeled with exactly one line!");
-                    }
-                    found = true;
-                }
-            }
-        }
-    }
-    return found;
-}
-
-
-bool FGAICarrier::mark_solid(ssgEntity* e, const list<string>& solid_objects, bool mark) {
-    bool found = false;
-    if (e->isAKindOf(ssgTypeBranch())) {
-        ssgBranch* br = (ssgBranch*)e;
-        ssgEntity* kid;
-
-        list<string>::const_iterator it;
-        for (it = solid_objects.begin(); it != solid_objects.end(); ++it)
-            mark = mark || (e->getName() && (*it) == e->getName());
-
-        for ( kid = br->getKid(0); kid != NULL ; kid = br->getNextKid() )
-            found = mark_solid(kid, solid_objects, mark) || found;
-
-        if (found)
-            br->setTraversalMaskBits(SSGTRAV_HOT);
-
-    } else if (e->isAKindOf(ssgTypeLeaf())) {
-        list<string>::const_iterator it;
-        for (it = solid_objects.begin(); it != solid_objects.end(); ++it) {
-            if (mark || (e->getName() && (*it) == e->getName())) {
-                e->setTraversalMaskBits(SSGTRAV_HOT);
-                ssgBase* ud = e->getUserData();
-
-                if (ud) {
-                    FGAICarrierHardware* ch = dynamic_cast<FGAICarrierHardware*>(ud);
-                    if (ch) {
-                        SG_LOG(SG_GENERAL, SG_WARN,
-                                "AICarrier: Carrier hardware gets marked twice!\n"
-                                "           You have probably a whole branch marked solid"
-                                " which also includes other carrier hardware.");
-                    } else {
-                        SG_LOG(SG_GENERAL, SG_ALERT,
-                                "AICarrier: Found user data attached to a leaf node which "
-                                "should be marked solid!\n    ****Skipping!****");
-                    }
-                } else {
-                    e->setUserData( FGAICarrierHardware::newSolid( this ) );
-                    found = true;
-                }
-            }
-        }
-    }
-    return found;
-}
-
-
-bool FGAICarrier::mark_cat(ssgEntity* e, const list<string>& cat_objects, bool mark) {
-    bool found = false;
-    if (e->isAKindOf(ssgTypeBranch())) {
-        ssgBranch* br = (ssgBranch*)e;
-        ssgEntity* kid;
-
-        list<string>::const_iterator it;
-        for (it = cat_objects.begin(); it != cat_objects.end(); ++it)
-            mark = mark || (e->getName() && (*it) == e->getName());
-
-        for ( kid = br->getKid(0); kid != NULL ; kid = br->getNextKid() )
-            found = mark_cat(kid, cat_objects, mark) || found;
-
-        if (found)
-            br->setTraversalMaskBits(SSGTRAV_HOT);
-
-    } else if (e->isAKindOf(ssgTypeLeaf())) {
-        list<string>::const_iterator it;
-        for (it = cat_objects.begin(); it != cat_objects.end(); ++it) {
-            if (mark || (e->getName() && (*it) == e->getName())) {
-                e->setTraversalMaskBits(SSGTRAV_HOT);
-                ssgBase* ud = e->getUserData();
-                if (ud) {
-                    FGAICarrierHardware* ch = dynamic_cast<FGAICarrierHardware*>(ud);
-                    if (ch) {
-                        SG_LOG(SG_GENERAL, SG_WARN,
-                                "AICarrier: Carrier hardware gets marked twice!\n"
-                                "You have probably a whole branch marked as"
-                                "a catapult which also includes other carrier hardware.");
-                    } else {
-                        SG_LOG(SG_GENERAL, SG_ALERT,
-                                "AICarrier: Found user data attached to a leaf node which "
-                                "should be marked as a catapult!\n    ****Skipping!****");
-                    }
-                } else {
-                    e->setUserData( FGAICarrierHardware::newCatapult( this ) );
-                    ssgLeaf *l = (ssgLeaf*)e;
-                    if ( l->getNumLines() != 1 ) {
-                        SG_LOG(SG_GENERAL, SG_ALERT,
-                                "AICarrier: Found a cat not modeled with exactly "
-                                "one line!");
-                    } else {
-                        // Now some special code to make sure the cat points in the right
-                        // direction. The 0 index must be the backward end, the 1 index
-                        // the forward end.
-                        // Forward is positive x-direction in our 3D model, also the model
-                        // as such is flattened when it is loaded, so we do not need to
-                        // care for transforms ...
-                        short v[2];
-                        l->getLine(0, v, v+1 );
-                        sgVec3 ends[2];
-                        for (int k=0; k<2; ++k)
-                            sgCopyVec3( ends[k], l->getVertex( v[k] ) );
-
-                        // When the 1 end is behind the 0 end, swap the coordinates.
-                        if (ends[0][0] < ends[1][0]) {
-                            sgCopyVec3( l->getVertex( v[0] ), ends[1] );
-                            sgCopyVec3( l->getVertex( v[1] ), ends[0] );
-                        }
-                        found = true;
-                    }
-                }
-            }
-        }
-    }
-    return found;
-}
-
 // find relative wind
 void FGAICarrier::UpdateWind( double dt) {
 
-    double recip;
-
-    //calculate the reciprocal hdg
-
-    if (hdg >= 180)
-        recip = hdg - 180;
-    else
-        recip = hdg + 180;
-
-    //cout <<" heading: " << hdg << "recip: " << recip << endl;
-
     //get the surface wind speed and direction
     wind_from_deg = _surface_wind_from_deg_node->getDoubleValue();
     wind_speed_kts  = _surface_wind_speed_node->getDoubleValue();
@@ -632,27 +387,21 @@ void FGAICarrier::UpdateWind( double dt) {
     + (rel_wind_speed_from_north_kts * rel_wind_speed_from_north_kts));
 
     //calculate the relative wind direction
-    rel_wind_from_deg = atan(rel_wind_speed_from_east_kts/rel_wind_speed_from_north_kts)
+    rel_wind_from_deg = atan2(rel_wind_speed_from_east_kts, rel_wind_speed_from_north_kts)
                             * SG_RADIANS_TO_DEGREES;
 
-    // rationalise the output
-    if (rel_wind_speed_from_north_kts <= 0) {
-        rel_wind_from_deg = 180 + rel_wind_from_deg;
-    } else {
-        if(rel_wind_speed_from_east_kts <= 0)
-            rel_wind_from_deg = 360 + rel_wind_from_deg;
-    }
-
     //calculate rel wind
     rel_wind = rel_wind_from_deg - hdg;
-    if (rel_wind > 180)
-        rel_wind -= 360;
+    SG_NORMALIZE_RANGE(rel_wind, -180.0, 180.0);
+
+    //set in to wind property
+    InToWind();
 
     //switch the wave-off lights
-    if (InToWind())
-       wave_off_lights = false;
-    else
-       wave_off_lights = true;
+    //if (InToWind())
+    //   wave_off_lights = false;
+    //else
+    //   wave_off_lights = true;
 
     // cout << "rel wind: " << rel_wind << endl;
 
@@ -661,18 +410,52 @@ void FGAICarrier::UpdateWind( double dt) {
 
 void FGAICarrier::TurnToLaunch(){
 
+    // calculate tgt heading
+    if (wind_speed_kts < 3){
+        tgt_heading = base_course;
+    } else {
+        tgt_heading = wind_from_deg;
+    }
+
     //calculate tgt speed
     double tgt_speed = 25 - wind_speed_kts;
     if (tgt_speed < 10)
         tgt_speed = 10;
 
     //turn the carrier
-    FGAIShip::TurnTo(wind_from_deg);
+    FGAIShip::TurnTo(tgt_heading);
     FGAIShip::AccelTo(tgt_speed);
 
 }
 
+void FGAICarrier::TurnToRecover(){
 
+    //these are the rules for adjusting heading to provide a relative wind
+    //down the angled flightdeck
+
+    if (wind_speed_kts < 3){
+        tgt_heading = base_course + 60;
+    } else if (rel_wind < -9 && rel_wind >= -180){
+        tgt_heading = wind_from_deg; 
+    } else if (rel_wind > -7 && rel_wind < 45){
+        tgt_heading = wind_from_deg + 60;
+    } else if (rel_wind >=45 && rel_wind < 180){
+        tgt_heading = wind_from_deg + 45;
+    } else 
+        tgt_heading = hdg;
+
+    SG_NORMALIZE_RANGE(tgt_heading, 0.0, 360.0);
+
+    //calculate tgt speed
+    double tgt_speed = 26 - wind_speed_kts;
+    if (tgt_speed < 10)
+        tgt_speed = 10;
+
+    //turn the carrier
+    FGAIShip::TurnTo(tgt_heading);
+    FGAIShip::AccelTo(tgt_speed);
+
+}
 void FGAICarrier::TurnToBase(){
 
     //turn the carrier
@@ -685,18 +468,8 @@ void FGAICarrier::TurnToBase(){
 void FGAICarrier::ReturnToBox(){
     double course, distance, az2;
 
-    //get the carrier position
-    carrierpos = pos;
-
-    //cout << "lat: " << carrierpos[1] << " lon: " << carrierpos[0] << endl;
-
     //calculate the bearing and range of the initial position from the carrier
-    geo_inverse_wgs_84(carrierpos[2],
-                       carrierpos[1],
-                       carrierpos[0],
-                       initialpos[1],
-                       initialpos[0],
-                       &course, &az2, &distance);
+    geo_inverse_wgs_84(pos, mOpBoxPos, &course, &az2, &distance);
 
     distance *= SG_METER_TO_NM;
 
@@ -720,33 +493,33 @@ bool FGAICarrier::OutsideBox() { //returns true if the carrier is outside operat
         return false;
     }
 
-    if (initialpos[1] >= 0) { //northern hemisphere
-        if (pos[1] >= initialpos[1] + max_lat)
+    if (mOpBoxPos.getLatitudeDeg() >= 0) { //northern hemisphere
+        if (pos.getLatitudeDeg() >= mOpBoxPos.getLatitudeDeg() + max_lat)
             return true;
 
-        if (pos[1] <= initialpos[1] - min_lat)
+        if (pos.getLatitudeDeg() <= mOpBoxPos.getLatitudeDeg() - min_lat)
             return true;
 
     } else {                  //southern hemisphere
-        if (pos[1] <= initialpos[1] - max_lat)
+        if (pos.getLatitudeDeg() <= mOpBoxPos.getLatitudeDeg() - max_lat)
             return true;
 
-        if (pos[1] >= initialpos[1] + min_lat)
+        if (pos.getLatitudeDeg() >= mOpBoxPos.getLatitudeDeg() + min_lat)
             return true;
     }
 
-    if (initialpos[0] >=0) { //eastern hemisphere
-        if (pos[0] >= initialpos[0] + max_long)
+    if (mOpBoxPos.getLongitudeDeg() >=0) { //eastern hemisphere
+        if (pos.getLongitudeDeg() >= mOpBoxPos.getLongitudeDeg() + max_long)
             return true;
 
-        if (pos[0] <= initialpos[0] - min_long)
+        if (pos.getLongitudeDeg() <= mOpBoxPos.getLongitudeDeg() - min_long)
             return true;
 
     } else {                 //western hemisphere
-        if (pos[0] <= initialpos[0] - max_long)
+        if (pos.getLongitudeDeg() <= mOpBoxPos.getLongitudeDeg() - max_long)
             return true;
 
-        if (pos[0] >= initialpos[0] + min_long)
+        if (pos.getLongitudeDeg() >= mOpBoxPos.getLongitudeDeg() + min_long)
             return true;
     }
 
@@ -756,16 +529,13 @@ bool FGAICarrier::OutsideBox() { //returns true if the carrier is outside operat
 } // end OutsideBox
 
 
-// return the distance to the horizon, given the altitude and the radius of the earth
-float FGAICarrier::Horizon(float h) {
-    return RADIUS_M * acos(RADIUS_M / (RADIUS_M + h));
-}
-
-
 bool FGAICarrier::InToWind() {
-    if ( fabs(rel_wind) < 5 )
-        return true;
+    in_to_wind = false;
 
+    if ( fabs(rel_wind) < 10 ){
+        in_to_wind = true;
+        return true;
+    }
     return false;
 }
 
@@ -845,7 +615,3 @@ void FGAICarrier::UpdateJBD(double dt, double jbd_transition_time) {
     return;
 
 } // end UpdateJBD
-
-
-int FGAICarrierHardware::unique_id = 1;
-
